
#include <glib.h>
#include <glib-object.h>
#include <libwnck/libwnck.h>
#include <stdlib.h>
#include <string.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gtk/gtk.h>
#include <gnome-menus/gmenu-tree.h>
#include <gio/gio.h>
#include <gobject/gvaluecollector.h>


#define GNOMENU_TYPE_APPLICATION (gnomenu_application_get_type ())
#define GNOMENU_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_APPLICATION, GnomenuApplication))
#define GNOMENU_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GNOMENU_TYPE_APPLICATION, GnomenuApplicationClass))
#define GNOMENU_IS_APPLICATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_APPLICATION))
#define GNOMENU_IS_APPLICATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOMENU_TYPE_APPLICATION))
#define GNOMENU_APPLICATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GNOMENU_TYPE_APPLICATION, GnomenuApplicationClass))

typedef struct _GnomenuApplication GnomenuApplication;
typedef struct _GnomenuApplicationClass GnomenuApplicationClass;
typedef struct _GnomenuApplicationPrivate GnomenuApplicationPrivate;
typedef struct _GnomenuParamSpecApplication GnomenuParamSpecApplication;

/******
 * The binary executable name is used as the key to link 
 * Wnck with GMenu.
 *
 * If the bin exec starts with wine, mono, python, use
 * the application's full name. (Although usually app and
 * dists don't agree on the same name!)
 *
 * the policy is sub-opt, because
 * one executable can actually be multiple applications,
 *
 * Example: NXClient Openoffice
 *
 * Known problem: Openoffice suite apps always show up 
 * as one Openoffice app (the last found one in GMenu).
 * */
struct _GnomenuApplication {
	GTypeInstance parent_instance;
	volatile int ref_count;
	GnomenuApplicationPrivate * priv;
	GList* wnck_applications;
};

struct _GnomenuApplicationClass {
	GTypeClass parent_class;
	void (*finalize) (GnomenuApplication *self);
};

struct _GnomenuApplicationPrivate {
	char* key;
	char* _readable_name;
	char* _exec_path;
	char* _icon_name;
	GdkPixbuf* _icon_pixbuf;
	gboolean _not_in_menu;
	GtkEventBox* _proxy_item;
	GtkLabel* name_widget;
	GtkLabel* status_widget;
	GtkImage* icon_widget;
};

struct _GnomenuParamSpecApplication {
	GParamSpec parent_instance;
};



gpointer gnomenu_application_ref (gpointer instance);
void gnomenu_application_unref (gpointer instance);
GParamSpec* gnomenu_param_spec_application (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void gnomenu_value_set_application (GValue* value, gpointer v_object);
gpointer gnomenu_value_get_application (const GValue* value);
GType gnomenu_application_get_type (void);
#define GNOMENU_APPLICATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_APPLICATION, GnomenuApplicationPrivate))
enum  {
	GNOMENU_APPLICATION_DUMMY_PROPERTY
};
static void _g_list_free_g_object_unref (GList* self);
extern GList* gnomenu_application_applications;
GList* gnomenu_application_applications = NULL;
extern GHashTable* gnomenu_application_dict;
GHashTable* gnomenu_application_dict = NULL;
static gboolean gnomenu_application_initialized;
static gboolean gnomenu_application_initialized = FALSE;
static void gnomenu_application_append_node_r (GMenuTreeDirectory* node);
static void gnomenu_application_application_opened (WnckScreen* s, WnckApplication* wapp);
static void _gnomenu_application_application_opened_wnck_screen_application_opened (WnckScreen* _sender, WnckApplication* app, gpointer self);
static void gnomenu_application_window_opened (WnckScreen* s, WnckWindow* wwin);
static void _gnomenu_application_window_opened_wnck_screen_window_opened (WnckScreen* _sender, WnckWindow* window, gpointer self);
static void gnomenu_application_window_closed (WnckScreen* s, WnckWindow* wwin);
static void _gnomenu_application_window_closed_wnck_screen_window_closed (WnckScreen* _sender, WnckWindow* window, gpointer self);
static void gnomenu_application_application_closed (WnckScreen* s, WnckApplication* wapp);
static void _gnomenu_application_application_closed_wnck_screen_application_closed (WnckScreen* _sender, WnckApplication* app, gpointer self);
void gnomenu_application_init (void);
static GnomenuApplication* gnomenu_application_new (void);
static GnomenuApplication* gnomenu_application_construct (GType object_type);
static GnomenuApplication* gnomenu_application_new (void);
GnomenuApplication* gnomenu_application_add_application (const char* key);
GnomenuApplication* gnomenu_application_lookup (const char* key);
static char* gnomenu_application_generate_key_from_wnck (WnckApplication* app);
static void gnomenu_application_set_not_in_menu (GnomenuApplication* self, gboolean value);
static void gnomenu_application_set_readable_name (GnomenuApplication* self, const char* value);
static void gnomenu_application_set_exec_path (GnomenuApplication* self, const char* value);
static void gnomenu_application_set_icon_name (GnomenuApplication* self, const char* value);
static void gnomenu_application_set_icon_pixbuf (GnomenuApplication* self, GdkPixbuf* value);
GnomenuApplication* gnomenu_application_lookup_from_wnck (WnckApplication* wapp);
GtkWidget* gnomenu_application_get_proxy_item (GnomenuApplication* self);
const char* gnomenu_application_get_readable_name (GnomenuApplication* self);
const char* gnomenu_application_get_icon_name (GnomenuApplication* self);
void gnomenu_application_update (GnomenuApplication* self);
static char* gnomenu_application_generate_key (GMenuTreeEntry* entry);
gboolean gnomenu_application_get_not_in_menu (GnomenuApplication* self);
static void gnomenu_application_create_proxy_item (GnomenuApplication* self);
char* get_task_name_by_pid (gint pid);
static char* gnomenu_application_get_process_name (WnckApplication* app);
const char* gnomenu_application_get_exec_path (GnomenuApplication* self);
GdkPixbuf* gnomenu_application_get_icon_pixbuf (GnomenuApplication* self);
static gpointer gnomenu_application_parent_class = NULL;
static void gnomenu_application_finalize (GnomenuApplication* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);



static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


static void _gnomenu_application_application_opened_wnck_screen_application_opened (WnckScreen* _sender, WnckApplication* app, gpointer self) {
	gnomenu_application_application_opened (_sender, app);
}


static void _gnomenu_application_window_opened_wnck_screen_window_opened (WnckScreen* _sender, WnckWindow* window, gpointer self) {
	gnomenu_application_window_opened (_sender, window);
}


static void _gnomenu_application_window_closed_wnck_screen_window_closed (WnckScreen* _sender, WnckWindow* window, gpointer self) {
	gnomenu_application_window_closed (_sender, window);
}


static void _gnomenu_application_application_closed_wnck_screen_application_closed (WnckScreen* _sender, WnckApplication* app, gpointer self) {
	gnomenu_application_application_closed (_sender, app);
}


void gnomenu_application_init (void) {
	GMenuTreeDirectory* node;
	GMenuTreeDirectory* _tmp1_;
	GMenuTreeDirectory* _tmp0_;
	GMenuTreeDirectory* _tmp3_;
	GMenuTreeDirectory* _tmp2_;
	WnckScreen* _tmp4_;
	WnckScreen* screen;
	GList* list;
	GList* appinfos;
	if (gnomenu_application_initialized) {
		return;
	}
	gnomenu_application_initialized = TRUE;
	node = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	node = (_tmp1_ = (_tmp0_ = gmenu_tree_get_root_directory (gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_INCLUDE_EXCLUDED)), (_tmp0_ == NULL) ? NULL : gmenu_tree_item_ref (_tmp0_)), (node == NULL) ? NULL : (node = (gmenu_tree_item_unref (node), NULL)), _tmp1_);
	gnomenu_application_append_node_r (node);
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	node = (_tmp3_ = (_tmp2_ = gmenu_tree_get_root_directory (gmenu_tree_lookup ("settings.menu", GMENU_TREE_FLAGS_INCLUDE_EXCLUDED)), (_tmp2_ == NULL) ? NULL : gmenu_tree_item_ref (_tmp2_)), (node == NULL) ? NULL : (node = (gmenu_tree_item_unref (node), NULL)), _tmp3_);
	gnomenu_application_append_node_r (node);
	_tmp4_ = NULL;
	screen = (_tmp4_ = wnck_screen_get_default (), (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_));
	list = wnck_screen_get_windows (screen);
	{
		GList* wwin_collection;
		GList* wwin_it;
		wwin_collection = list;
		for (wwin_it = wwin_collection; wwin_it != NULL; wwin_it = wwin_it->next) {
			WnckWindow* _tmp6_;
			WnckWindow* wwin;
			_tmp6_ = NULL;
			wwin = (_tmp6_ = (WnckWindow*) wwin_it->data, (_tmp6_ == NULL) ? NULL : g_object_ref (_tmp6_));
			{
				WnckApplication* _tmp5_;
				WnckApplication* wapp;
				_tmp5_ = NULL;
				wapp = (_tmp5_ = wnck_window_get_application (wwin), (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_));
				gnomenu_application_application_opened (screen, wapp);
				(wwin == NULL) ? NULL : (wwin = (g_object_unref (wwin), NULL));
				(wapp == NULL) ? NULL : (wapp = (g_object_unref (wapp), NULL));
			}
		}
	}
	g_signal_connect (screen, "application-opened", (GCallback) _gnomenu_application_application_opened_wnck_screen_application_opened, NULL);
	g_signal_connect (screen, "window-opened", (GCallback) _gnomenu_application_window_opened_wnck_screen_window_opened, NULL);
	g_signal_connect (screen, "window-closed", (GCallback) _gnomenu_application_window_closed_wnck_screen_window_closed, NULL);
	g_signal_connect (screen, "application-closed", (GCallback) _gnomenu_application_application_closed_wnck_screen_application_closed, NULL);
	appinfos = g_app_info_get_all ();
	{
		GList* appinfo_collection;
		GList* appinfo_it;
		appinfo_collection = appinfos;
		for (appinfo_it = appinfo_collection; appinfo_it != NULL; appinfo_it = appinfo_it->next) {
			GAppInfo* _tmp7_;
			GAppInfo* appinfo;
			_tmp7_ = NULL;
			appinfo = (_tmp7_ = (GAppInfo*) appinfo_it->data, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_));
			{
				g_message ("application.vala:86: %s", g_app_info_get_executable (appinfo));
				(appinfo == NULL) ? NULL : (appinfo = (g_object_unref (appinfo), NULL));
			}
		}
	}
	(node == NULL) ? NULL : (node = (gmenu_tree_item_unref (node), NULL));
	(screen == NULL) ? NULL : (screen = (g_object_unref (screen), NULL));
}


static GnomenuApplication* gnomenu_application_construct (GType object_type) {
	GnomenuApplication* self;
	self = (GnomenuApplication*) g_type_create_instance (object_type);
	return self;
}


static GnomenuApplication* gnomenu_application_new (void) {
	return gnomenu_application_construct (GNOMENU_TYPE_APPLICATION);
}


GnomenuApplication* gnomenu_application_add_application (const char* key) {
	GnomenuApplication* app;
	GnomenuApplication* rt;
	const char* _tmp0_;
	GnomenuApplication* _tmp1_;
	GnomenuApplication* _tmp2_;
	g_return_val_if_fail (key != NULL, NULL);
	app = gnomenu_application_new ();
	rt = app;
	_tmp0_ = NULL;
	g_hash_table_insert (gnomenu_application_dict, (_tmp0_ = key, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), app);
	_tmp1_ = NULL;
	gnomenu_application_applications = g_list_prepend (gnomenu_application_applications, (_tmp1_ = app, app = NULL, _tmp1_));
	_tmp2_ = NULL;
	return (_tmp2_ = rt, (app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL)), _tmp2_);
}


GnomenuApplication* gnomenu_application_lookup (const char* key) {
	g_return_val_if_fail (key != NULL, NULL);
	gnomenu_application_init ();
	return (GnomenuApplication*) g_hash_table_lookup (gnomenu_application_dict, key);
}


GnomenuApplication* gnomenu_application_lookup_from_wnck (WnckApplication* wapp) {
	char* key;
	GnomenuApplication* rt;
	GnomenuApplication* _tmp4_;
	g_return_val_if_fail (wapp != NULL, NULL);
	gnomenu_application_init ();
	key = gnomenu_application_generate_key_from_wnck (wapp);
	g_debug ("application.vala:107: wnck key = %s", key);
	rt = (GnomenuApplication*) g_hash_table_lookup (gnomenu_application_dict, key);
	if (rt == NULL) {
		GnomenuApplication* app;
		char* _tmp1_;
		const char* _tmp0_;
		const char* _tmp2_;
		GnomenuApplication* _tmp3_;
		app = gnomenu_application_new ();
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		app->priv->key = (_tmp1_ = (_tmp0_ = key, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), app->priv->key = (g_free (app->priv->key), NULL), _tmp1_);
		gnomenu_application_set_not_in_menu (app, TRUE);
		gnomenu_application_set_readable_name (app, wnck_application_get_name (wapp));
		gnomenu_application_set_exec_path (app, NULL);
		/* NOTE: get_icon_name is not implement in wnck.
		 * Therefore we set icon_pixbuf, which
		 * indeed has a higher priority in switcher.vala
		 * */
		gnomenu_application_set_icon_name (app, wnck_application_get_icon_name (wapp));
		_tmp2_ = NULL;
		g_hash_table_insert (gnomenu_application_dict, (_tmp2_ = key, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), app);
		rt = app;
		_tmp3_ = NULL;
		gnomenu_application_applications = g_list_prepend (gnomenu_application_applications, (_tmp3_ = app, app = NULL, _tmp3_));
		(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
	}
	/* Always use the icon_pixbuf obtained from wnck.*/
	gnomenu_application_set_icon_pixbuf (rt, wnck_application_get_mini_icon (wapp));
	_tmp4_ = NULL;
	return (_tmp4_ = rt, key = (g_free (key), NULL), _tmp4_);
}


void gnomenu_application_update (GnomenuApplication* self) {
	g_return_if_fail (self != NULL);
	if (gnomenu_application_get_proxy_item (self) == NULL) {
		return;
	}
	if (self->wnck_applications != NULL) {
		gint n;
		char* _tmp1_;
		n = 0;
		{
			GList* wapp_collection;
			GList* wapp_it;
			wapp_collection = self->wnck_applications;
			for (wapp_it = wapp_collection; wapp_it != NULL; wapp_it = wapp_it->next) {
				WnckApplication* _tmp0_;
				WnckApplication* wapp;
				_tmp0_ = NULL;
				wapp = (_tmp0_ = (WnckApplication*) wapp_it->data, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
				{
					n = n + wnck_application_get_n_windows (wapp);
					(wapp == NULL) ? NULL : (wapp = (g_object_unref (wapp), NULL));
				}
			}
		}
		_tmp1_ = NULL;
		gtk_label_set_label (self->priv->status_widget, _tmp1_ = g_strdup_printf ("%u instances, %d windows", g_list_length (self->wnck_applications), n));
		_tmp1_ = (g_free (_tmp1_), NULL);
	} else {
		gtk_label_set_label (self->priv->status_widget, "not launched");
	}
	gtk_label_set_label (self->priv->name_widget, self->priv->_readable_name);
	g_object_set (self->priv->icon_widget, "icon-name", self->priv->_icon_name, NULL);
}


static void gnomenu_application_append_node_r (GMenuTreeDirectory* node) {
	g_return_if_fail (node != NULL);
	{
		GSList* item_collection;
		GSList* item_it;
		item_collection = gmenu_tree_directory_get_contents (node);
		for (item_it = item_collection; item_it != NULL; item_it = item_it->next) {
			GMenuTreeItem* _tmp5_;
			GMenuTreeItem* item;
			_tmp5_ = NULL;
			item = (_tmp5_ = (GMenuTreeItem*) item_it->data, (_tmp5_ == NULL) ? NULL : gmenu_tree_item_ref (_tmp5_));
			{
				switch (gmenu_tree_item_get_type (item)) {
					case GMENU_TREE_ITEM_ENTRY:
					{
						GMenuTreeEntry* _tmp0_;
						GMenuTreeEntry* entry;
						char* key;
						GnomenuApplication* app;
						char* _tmp2_;
						const char* _tmp1_;
						const char* _tmp3_;
						GnomenuApplication* _tmp4_;
						_tmp0_ = NULL;
						entry = (_tmp0_ = (GMenuTreeEntry*) item, (_tmp0_ == NULL) ? NULL : gmenu_tree_item_ref (_tmp0_));
						key = gnomenu_application_generate_key (entry);
						g_debug ("application.vala:151: gmenu key = %s", key);
						app = gnomenu_application_new ();
						_tmp2_ = NULL;
						_tmp1_ = NULL;
						app->priv->key = (_tmp2_ = (_tmp1_ = key, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), app->priv->key = (g_free (app->priv->key), NULL), _tmp2_);
						gnomenu_application_set_not_in_menu (app, FALSE);
						gnomenu_application_set_readable_name (app, gmenu_tree_entry_get_name (entry));
						gnomenu_application_set_exec_path (app, gmenu_tree_entry_get_exec (entry));
						gnomenu_application_set_icon_name (app, gmenu_tree_entry_get_icon (entry));
						_tmp3_ = NULL;
						g_hash_table_insert (gnomenu_application_dict, (_tmp3_ = key, (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), app);
						_tmp4_ = NULL;
						gnomenu_application_applications = g_list_prepend (gnomenu_application_applications, (_tmp4_ = app, app = NULL, _tmp4_));
						(entry == NULL) ? NULL : (entry = (gmenu_tree_item_unref (entry), NULL));
						key = (g_free (key), NULL);
						(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
						break;
					}
					case GMENU_TREE_ITEM_DIRECTORY:
					{
						gnomenu_application_append_node_r ((GMenuTreeDirectory*) item);
						break;
					}
				}
				(item == NULL) ? NULL : (item = (gmenu_tree_item_unref (item), NULL));
			}
		}
	}
}


static void gnomenu_application_window_opened (WnckScreen* s, WnckWindow* wwin) {
	GnomenuApplication* _tmp0_;
	GnomenuApplication* app;
	g_return_if_fail (s != NULL);
	g_return_if_fail (wwin != NULL);
	_tmp0_ = NULL;
	app = (_tmp0_ = gnomenu_application_lookup_from_wnck (wnck_window_get_application (wwin)), (_tmp0_ == NULL) ? NULL : gnomenu_application_ref (_tmp0_));
	g_object_set_data ((GObject*) wwin, "gnomenu-app", app);
	gnomenu_application_update (app);
	(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
}


static void gnomenu_application_window_closed (WnckScreen* s, WnckWindow* wwin) {
	GnomenuApplication* _tmp1_;
	void* _tmp0_;
	GnomenuApplication* app;
	g_return_if_fail (s != NULL);
	g_return_if_fail (wwin != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	app = (_tmp1_ = (_tmp0_ = g_object_get_data ((GObject*) wwin, "gnomenu-app"), GNOMENU_IS_APPLICATION (_tmp0_) ? ((GnomenuApplication*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : gnomenu_application_ref (_tmp1_));
	if (app == NULL) {
		(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
		return;
	}
	gnomenu_application_update (app);
	(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
}


static void gnomenu_application_application_opened (WnckScreen* s, WnckApplication* wapp) {
	GnomenuApplication* _tmp0_;
	GnomenuApplication* app;
	WnckApplication* _tmp1_;
	g_return_if_fail (s != NULL);
	g_return_if_fail (wapp != NULL);
	_tmp0_ = NULL;
	app = (_tmp0_ = gnomenu_application_lookup_from_wnck (wapp), (_tmp0_ == NULL) ? NULL : gnomenu_application_ref (_tmp0_));
	_tmp1_ = NULL;
	app->wnck_applications = g_list_prepend (app->wnck_applications, (_tmp1_ = wapp, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)));
	gnomenu_application_update (app);
	(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
}


static void gnomenu_application_application_closed (WnckScreen* s, WnckApplication* wapp) {
	g_return_if_fail (s != NULL);
	g_return_if_fail (wapp != NULL);
	{
		GList* app_collection;
		GList* app_it;
		app_collection = gnomenu_application_applications;
		for (app_it = app_collection; app_it != NULL; app_it = app_it->next) {
			GnomenuApplication* _tmp1_;
			GnomenuApplication* app;
			_tmp1_ = NULL;
			app = (_tmp1_ = (GnomenuApplication*) app_it->data, (_tmp1_ == NULL) ? NULL : gnomenu_application_ref (_tmp1_));
			{
				gboolean _tmp0_;
				/*Don't use lookup_from_wnck, since no pid now?*/
				if (g_list_find (app->wnck_applications, wapp) != NULL) {
					app->wnck_applications = g_list_remove (app->wnck_applications, wapp);
					gnomenu_application_update (app);
				}
				_tmp0_ = FALSE;
				if (app->wnck_applications == NULL) {
					_tmp0_ = app->priv->_not_in_menu;
				} else {
					_tmp0_ = FALSE;
				}
				if (_tmp0_ = TRUE) {
					/*TODO: After merging the code to Monitor, fire a signal*/
					g_hash_table_remove (gnomenu_application_dict, app->priv->key);
					gnomenu_application_applications = g_list_remove (gnomenu_application_applications, app);
				}
				(app == NULL) ? NULL : (app = (gnomenu_application_unref (app), NULL));
			}
		}
	}
}


static void gnomenu_application_create_proxy_item (GnomenuApplication* self) {
	GtkEventBox* _tmp0_;
	GtkLabel* _tmp1_;
	GtkLabel* _tmp2_;
	GtkImage* _tmp3_;
	GtkHBox* hbox;
	GtkVBox* vbox;
	g_return_if_fail (self != NULL);
	_tmp0_ = NULL;
	self->priv->_proxy_item = (_tmp0_ = g_object_ref_sink ((GtkEventBox*) gtk_event_box_new ()), (self->priv->_proxy_item == NULL) ? NULL : (self->priv->_proxy_item = (g_object_unref (self->priv->_proxy_item), NULL)), _tmp0_);
	_tmp1_ = NULL;
	self->priv->name_widget = (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->name_widget == NULL) ? NULL : (self->priv->name_widget = (g_object_unref (self->priv->name_widget), NULL)), _tmp1_);
	_tmp2_ = NULL;
	self->priv->status_widget = (_tmp2_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("")), (self->priv->status_widget == NULL) ? NULL : (self->priv->status_widget = (g_object_unref (self->priv->status_widget), NULL)), _tmp2_);
	_tmp3_ = NULL;
	self->priv->icon_widget = (_tmp3_ = g_object_ref_sink ((GtkImage*) gtk_image_new ()), (self->priv->icon_widget == NULL) ? NULL : (self->priv->icon_widget = (g_object_unref (self->priv->icon_widget), NULL)), _tmp3_);
	hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 0));
	vbox = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 0));
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->icon_widget, FALSE, FALSE, (guint) 0);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) vbox, TRUE, TRUE, (guint) 0);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) self->priv->name_widget, FALSE, FALSE, (guint) 0);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) self->priv->status_widget, FALSE, FALSE, (guint) 0);
	gtk_container_add ((GtkContainer*) self->priv->_proxy_item, (GtkWidget*) hbox);
	gnomenu_application_update (self);
	(hbox == NULL) ? NULL : (hbox = (g_object_unref (hbox), NULL));
	(vbox == NULL) ? NULL : (vbox = (g_object_unref (vbox), NULL));
}


static char* gnomenu_application_generate_key_from_wnck (WnckApplication* app) {
	g_return_val_if_fail (app != NULL, NULL);
	return gnomenu_application_get_process_name (app);
}


static char* gnomenu_application_get_process_name (WnckApplication* app) {
	GError * _inner_error_;
	char* cmdline;
	gint args_size;
	gint args_length1;
	char** args;
	gboolean _tmp0_;
	g_return_val_if_fail (app != NULL, NULL);
	_inner_error_ = NULL;
	cmdline = get_task_name_by_pid (wnck_application_get_pid (app));
	args = (args_length1 = 0, NULL);
	_tmp0_ = FALSE;
	if (cmdline == NULL) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = _vala_strcmp0 (cmdline, "") == 0;
	}
	if (_tmp0_) {
		const char* _tmp1_;
		char* _tmp2_;
		_tmp1_ = NULL;
		_tmp2_ = NULL;
		return (_tmp2_ = (_tmp1_ = wnck_application_get_name (app), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), cmdline = (g_free (cmdline), NULL), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), _tmp2_);
	}
	{
		char* basename;
		GQuark _tmp7_;
		const char* _tmp6_;
		char* _tmp8_;
		g_shell_parse_argv (cmdline, &args_length1, &args, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch3_g_error;
			goto __finally3;
		}
		basename = g_path_get_basename (args[0]);
		_tmp6_ = NULL;
		static GQuark _tmp7__label0 = 0;
		static GQuark _tmp7__label1 = 0;
		static GQuark _tmp7__label2 = 0;
		static GQuark _tmp7__label3 = 0;
		static GQuark _tmp7__label4 = 0;
		_tmp6_ = basename;
		_tmp7_ = (NULL == _tmp6_) ? 0 : g_quark_from_string (_tmp6_);
		if ((((_tmp7_ == ((0 != _tmp7__label0) ? _tmp7__label0 : (_tmp7__label0 = g_quark_from_static_string ("mono")))) || (_tmp7_ == ((0 != _tmp7__label1) ? _tmp7__label1 : (_tmp7__label1 = g_quark_from_static_string ("python"))))) || (_tmp7_ == ((0 != _tmp7__label2) ? _tmp7__label2 : (_tmp7__label2 = g_quark_from_static_string ("python2.5"))))) || (_tmp7_ == ((0 != _tmp7__label3) ? _tmp7__label3 : (_tmp7__label3 = g_quark_from_static_string ("wine")))))
		do {
			const char* _tmp3_;
			char* _tmp4_;
			_tmp3_ = NULL;
			_tmp4_ = NULL;
			return (_tmp4_ = (_tmp3_ = wnck_application_get_name (app), (_tmp3_ == NULL) ? NULL : g_strdup (_tmp3_)), basename = (g_free (basename), NULL), cmdline = (g_free (cmdline), NULL), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), _tmp4_);
		} while (0); else if (_tmp7_ == ((0 != _tmp7__label4) ? _tmp7__label4 : (_tmp7__label4 = g_quark_from_static_string ("swriter.bin"))))
		do {
			char* _tmp5_;
			_tmp5_ = NULL;
			return (_tmp5_ = g_strdup ("openoffice.org"), basename = (g_free (basename), NULL), cmdline = (g_free (cmdline), NULL), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), _tmp5_);
		} while (0);
		_tmp8_ = NULL;
		return (_tmp8_ = basename, cmdline = (g_free (cmdline), NULL), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), _tmp8_);
	}
	goto __finally3;
	__catch3_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			const char* _tmp9_;
			char* _tmp10_;
			_tmp9_ = NULL;
			_tmp10_ = NULL;
			return (_tmp10_ = (_tmp9_ = wnck_application_get_name (app), (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), cmdline = (g_free (cmdline), NULL), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), _tmp10_);
		}
	}
	__finally3:
	if (_inner_error_ != NULL) {
		cmdline = (g_free (cmdline), NULL);
		args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	cmdline = (g_free (cmdline), NULL);
	args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
}


/* FIXME: 
 * The following functions are 
 * to be replaced by a manually written normalizer with StringBuilder*/
static char* gnomenu_application_generate_key (GMenuTreeEntry* entry) {
	GError * _inner_error_;
	gint args_size;
	gint args_length1;
	char** args;
	GString* sb;
	gboolean exec;
	g_return_val_if_fail (entry != NULL, NULL);
	_inner_error_ = NULL;
	args = (args_length1 = 0, NULL);
	sb = g_string_new ("");
	exec = TRUE;
	{
		const char* _tmp4_;
		char* _tmp5_;
		g_shell_parse_argv (gmenu_tree_entry_get_exec (entry), &args_length1, &args, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch4_g_error;
			goto __finally4;
		}
		{
			gint i;
			i = 0;
			for (; i < args_length1; i++) {
				if (_vala_strcmp0 (args[i], "env") == 0) {
					while (g_utf8_strchr (args[i + 1], (glong) (-1), (gunichar) '=') != NULL) {
						i++;
					}
					continue;
				}
				if (exec) {
					char* basename;
					GQuark _tmp3_;
					const char* _tmp2_;
					basename = g_path_get_basename (args[i]);
					_tmp2_ = NULL;
					static GQuark _tmp3__label0 = 0;
					static GQuark _tmp3__label1 = 0;
					static GQuark _tmp3__label2 = 0;
					static GQuark _tmp3__label3 = 0;
					_tmp2_ = basename;
					_tmp3_ = (NULL == _tmp2_) ? 0 : g_quark_from_string (_tmp2_);
					if ((((_tmp3_ == ((0 != _tmp3__label0) ? _tmp3__label0 : (_tmp3__label0 = g_quark_from_static_string ("mono")))) || (_tmp3_ == ((0 != _tmp3__label1) ? _tmp3__label1 : (_tmp3__label1 = g_quark_from_static_string ("wine"))))) || (_tmp3_ == ((0 != _tmp3__label2) ? _tmp3__label2 : (_tmp3__label2 = g_quark_from_static_string ("python"))))) || (_tmp3_ == ((0 != _tmp3__label3) ? _tmp3__label3 : (_tmp3__label3 = g_quark_from_static_string ("python2.5")))))
					do {
						const char* _tmp0_;
						char* _tmp1_;
						_tmp0_ = NULL;
						_tmp1_ = NULL;
						return (_tmp1_ = (_tmp0_ = gmenu_tree_entry_get_name (entry), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), basename = (g_free (basename), NULL), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp1_);
					} while (0);
					g_string_append (sb, basename);
					exec = FALSE;
					basename = (g_free (basename), NULL);
				} else {
				}
			}
		}
		/*ignore other parameters */
		_tmp4_ = NULL;
		_tmp5_ = NULL;
		return (_tmp5_ = (_tmp4_ = sb->str, (_tmp4_ == NULL) ? NULL : g_strdup (_tmp4_)), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp5_);
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			const char* _tmp6_;
			char* _tmp7_;
			_tmp6_ = NULL;
			_tmp7_ = NULL;
			return (_tmp7_ = (_tmp6_ = gmenu_tree_entry_get_exec (entry), (_tmp6_ == NULL) ? NULL : g_strdup (_tmp6_)), (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), _tmp7_);
		}
	}
	__finally4:
	if (_inner_error_ != NULL) {
		args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
		(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	args = (_vala_array_free (args, args_length1, (GDestroyNotify) g_free), NULL);
	(sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL));
}


const char* gnomenu_application_get_readable_name (GnomenuApplication* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_readable_name;
}


static void gnomenu_application_set_readable_name (GnomenuApplication* self, const char* value) {
	char* _tmp2_;
	const char* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_readable_name = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), self->priv->_readable_name = (g_free (self->priv->_readable_name), NULL), _tmp2_);
}


const char* gnomenu_application_get_exec_path (GnomenuApplication* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_exec_path;
}


static void gnomenu_application_set_exec_path (GnomenuApplication* self, const char* value) {
	char* _tmp2_;
	const char* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_exec_path = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), self->priv->_exec_path = (g_free (self->priv->_exec_path), NULL), _tmp2_);
}


const char* gnomenu_application_get_icon_name (GnomenuApplication* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_icon_name;
}


static void gnomenu_application_set_icon_name (GnomenuApplication* self, const char* value) {
	char* _tmp2_;
	const char* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_icon_name = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), self->priv->_icon_name = (g_free (self->priv->_icon_name), NULL), _tmp2_);
}


GdkPixbuf* gnomenu_application_get_icon_pixbuf (GnomenuApplication* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_icon_pixbuf;
}


static void gnomenu_application_set_icon_pixbuf (GnomenuApplication* self, GdkPixbuf* value) {
	GdkPixbuf* _tmp2_;
	GdkPixbuf* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_icon_pixbuf = (_tmp2_ = (_tmp1_ = value, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_icon_pixbuf == NULL) ? NULL : (self->priv->_icon_pixbuf = (g_object_unref (self->priv->_icon_pixbuf), NULL)), _tmp2_);
}


gboolean gnomenu_application_get_not_in_menu (GnomenuApplication* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_not_in_menu;
}


static void gnomenu_application_set_not_in_menu (GnomenuApplication* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_not_in_menu = value;
}


GtkWidget* gnomenu_application_get_proxy_item (GnomenuApplication* self) {
	g_return_val_if_fail (self != NULL, NULL);
	if (self->priv->_proxy_item == NULL) {
		gnomenu_application_create_proxy_item (self);
	}
	return (GtkWidget*) self->priv->_proxy_item;
}


static void gnomenu_value_application_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void gnomenu_value_application_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		gnomenu_application_unref (value->data[0].v_pointer);
	}
}


static void gnomenu_value_application_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = gnomenu_application_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer gnomenu_value_application_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* gnomenu_value_application_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		GnomenuApplication* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = gnomenu_application_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* gnomenu_value_application_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	GnomenuApplication** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = gnomenu_application_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* gnomenu_param_spec_application (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	GnomenuParamSpecApplication* spec;
	g_return_val_if_fail (g_type_is_a (object_type, GNOMENU_TYPE_APPLICATION), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer gnomenu_value_get_application (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GNOMENU_TYPE_APPLICATION), NULL);
	return value->data[0].v_pointer;
}


void gnomenu_value_set_application (GValue* value, gpointer v_object) {
	GnomenuApplication* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, GNOMENU_TYPE_APPLICATION));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, GNOMENU_TYPE_APPLICATION));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		gnomenu_application_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		gnomenu_application_unref (old);
	}
}


static void gnomenu_application_class_init (GnomenuApplicationClass * klass) {
	gnomenu_application_parent_class = g_type_class_peek_parent (klass);
	GNOMENU_APPLICATION_CLASS (klass)->finalize = gnomenu_application_finalize;
	g_type_class_add_private (klass, sizeof (GnomenuApplicationPrivate));
	gnomenu_application_dict = g_hash_table_new (g_str_hash, g_str_equal);
}


static void gnomenu_application_instance_init (GnomenuApplication * self) {
	self->priv = GNOMENU_APPLICATION_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void gnomenu_application_finalize (GnomenuApplication* obj) {
	GnomenuApplication * self;
	self = GNOMENU_APPLICATION (obj);
	self->priv->key = (g_free (self->priv->key), NULL);
	self->priv->_readable_name = (g_free (self->priv->_readable_name), NULL);
	self->priv->_exec_path = (g_free (self->priv->_exec_path), NULL);
	self->priv->_icon_name = (g_free (self->priv->_icon_name), NULL);
	(self->priv->_icon_pixbuf == NULL) ? NULL : (self->priv->_icon_pixbuf = (g_object_unref (self->priv->_icon_pixbuf), NULL));
	(self->wnck_applications == NULL) ? NULL : (self->wnck_applications = (_g_list_free_g_object_unref (self->wnck_applications), NULL));
	(self->priv->_proxy_item == NULL) ? NULL : (self->priv->_proxy_item = (g_object_unref (self->priv->_proxy_item), NULL));
	(self->priv->name_widget == NULL) ? NULL : (self->priv->name_widget = (g_object_unref (self->priv->name_widget), NULL));
	(self->priv->status_widget == NULL) ? NULL : (self->priv->status_widget = (g_object_unref (self->priv->status_widget), NULL));
	(self->priv->icon_widget == NULL) ? NULL : (self->priv->icon_widget = (g_object_unref (self->priv->icon_widget), NULL));
}


GType gnomenu_application_get_type (void) {
	static GType gnomenu_application_type_id = 0;
	if (gnomenu_application_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { gnomenu_value_application_init, gnomenu_value_application_free_value, gnomenu_value_application_copy_value, gnomenu_value_application_peek_pointer, "p", gnomenu_value_application_collect_value, "p", gnomenu_value_application_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuApplicationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_application_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuApplication), 0, (GInstanceInitFunc) gnomenu_application_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		gnomenu_application_type_id = g_type_register_fundamental (g_type_fundamental_next (), "GnomenuApplication", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return gnomenu_application_type_id;
}


gpointer gnomenu_application_ref (gpointer instance) {
	GnomenuApplication* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void gnomenu_application_unref (gpointer instance) {
	GnomenuApplication* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		GNOMENU_APPLICATION_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




