
#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define GNOMENU_TYPE_ITEM_TYPE (gnomenu_item_type_get_type ())

#define GNOMENU_TYPE_ITEM_STATE (gnomenu_item_state_get_type ())

#define GNOMENU_TYPE_ITEM (gnomenu_item_get_type ())
#define GNOMENU_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_ITEM, GnomenuItem))
#define GNOMENU_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_ITEM))
#define GNOMENU_ITEM_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GNOMENU_TYPE_ITEM, GnomenuItemIface))

typedef struct _GnomenuItem GnomenuItem;
typedef struct _GnomenuItemIface GnomenuItemIface;

#define GNOMENU_TYPE_SHELL (gnomenu_shell_get_type ())
#define GNOMENU_SHELL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_SHELL, GnomenuShell))
#define GNOMENU_IS_SHELL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_SHELL))
#define GNOMENU_SHELL_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GNOMENU_TYPE_SHELL, GnomenuShellIface))

typedef struct _GnomenuShell GnomenuShell;
typedef struct _GnomenuShellIface GnomenuShellIface;

typedef enum  {
	GNOMENU_ITEM_TYPE_NORMAL = 0,
	GNOMENU_ITEM_TYPE_CHECK = 1,
	GNOMENU_ITEM_TYPE_RADIO = 2,
	GNOMENU_ITEM_TYPE_IMAGE = 3,
	GNOMENU_ITEM_TYPE_SEPARATOR = 4,
	GNOMENU_ITEM_TYPE_ARROW = 5,
	GNOMENU_ITEM_TYPE_ICON = 6
} GnomenuItemType;

typedef enum  {
	GNOMENU_ITEM_STATE_UNTOGGLED = 0,
	GNOMENU_ITEM_STATE_TOGGLED = 1,
	GNOMENU_ITEM_STATE_TRISTATE = 2
} GnomenuItemState;

struct _GnomenuShellIface {
	GTypeInterface parent_iface;
	GnomenuItem* (*get_item) (GnomenuShell* self, gint position);
	GnomenuItem* (*get_item_by_id) (GnomenuShell* self, const char* id);
	gint (*get_item_position) (GnomenuShell* self, GnomenuItem* item);
	GnomenuItem* (*get_owner) (GnomenuShell* self);
	gint (*get_length) (GnomenuShell* self);
	void (*set_length) (GnomenuShell* self, gint value);
};

struct _GnomenuItemIface {
	GTypeInterface parent_iface;
	GnomenuShell* (*get_shell) (GnomenuItem* self);
	GnomenuShell* (*get_sub_shell) (GnomenuItem* self);
	gboolean (*get_has_sub_shell) (GnomenuItem* self);
	void (*set_has_sub_shell) (GnomenuItem* self, gboolean value);
	gboolean (*get_client_side_sub_shell) (GnomenuItem* self);
	void (*set_client_side_sub_shell) (GnomenuItem* self, gboolean value);
	const char* (*get_item_id) (GnomenuItem* self);
	void (*set_item_id) (GnomenuItem* self, const char* value);
	GnomenuItemType (*get_item_type) (GnomenuItem* self);
	void (*set_item_type) (GnomenuItem* self, GnomenuItemType value);
	gboolean (*get_item_use_underline) (GnomenuItem* self);
	void (*set_item_use_underline) (GnomenuItem* self, gboolean value);
	gboolean (*get_item_sensitive) (GnomenuItem* self);
	void (*set_item_sensitive) (GnomenuItem* self, gboolean value);
	gboolean (*get_item_visible) (GnomenuItem* self);
	void (*set_item_visible) (GnomenuItem* self, gboolean value);
	GnomenuItemState (*get_item_state) (GnomenuItem* self);
	void (*set_item_state) (GnomenuItem* self, GnomenuItemState value);
	const char* (*get_item_label) (GnomenuItem* self);
	void (*set_item_label) (GnomenuItem* self, const char* value);
	const char* (*get_item_icon) (GnomenuItem* self);
	void (*set_item_icon) (GnomenuItem* self, const char* value);
	const char* (*get_item_accel_text) (GnomenuItem* self);
	void (*set_item_accel_text) (GnomenuItem* self, const char* value);
	const char* (*get_item_font) (GnomenuItem* self);
	void (*set_item_font) (GnomenuItem* self, const char* value);
};



GType gnomenu_item_type_get_type (void);
GType gnomenu_item_state_get_type (void);
GType gnomenu_shell_get_type (void);
GType gnomenu_item_get_type (void);
GnomenuShell* gnomenu_item_get_shell (GnomenuItem* self);
GnomenuItem* gnomenu_shell_get_owner (GnomenuShell* self);
gboolean gnomenu_item_is_child_of (GnomenuItem* self, GnomenuItem* possible_parent);
GnomenuItemState gnomenu_item_state_from_string (const char* str);
const char* gnomenu_item_state_to_string (GnomenuItemState state);
GnomenuItemType gnomenu_item_type_from_string (const char* str);
gboolean gnomenu_item_type_has_label (GnomenuItemType type);
const char* gnomenu_item_type_to_string (GnomenuItemType type);
GnomenuShell* gnomenu_item_get_sub_shell (GnomenuItem* self);
gboolean gnomenu_item_get_has_sub_shell (GnomenuItem* self);
void gnomenu_item_set_has_sub_shell (GnomenuItem* self, gboolean value);
gboolean gnomenu_item_get_client_side_sub_shell (GnomenuItem* self);
void gnomenu_item_set_client_side_sub_shell (GnomenuItem* self, gboolean value);
GnomenuShell* gnomenu_shell_get_toplevel_shell (GnomenuShell* self);
GnomenuShell* gnomenu_item_get_toplevel_shell (GnomenuItem* self);
gint gnomenu_shell_get_item_position (GnomenuShell* self, GnomenuItem* item);
gint gnomenu_item_get_item_position (GnomenuItem* self);
const char* gnomenu_item_get_item_id (GnomenuItem* self);
void gnomenu_item_set_item_id (GnomenuItem* self, const char* value);
GnomenuItemType gnomenu_item_get_item_type (GnomenuItem* self);
void gnomenu_item_set_item_type (GnomenuItem* self, GnomenuItemType value);
gboolean gnomenu_item_get_item_use_underline (GnomenuItem* self);
void gnomenu_item_set_item_use_underline (GnomenuItem* self, gboolean value);
gboolean gnomenu_item_get_item_sensitive (GnomenuItem* self);
void gnomenu_item_set_item_sensitive (GnomenuItem* self, gboolean value);
gboolean gnomenu_item_get_item_visible (GnomenuItem* self);
void gnomenu_item_set_item_visible (GnomenuItem* self, gboolean value);
GnomenuItemState gnomenu_item_get_item_state (GnomenuItem* self);
void gnomenu_item_set_item_state (GnomenuItem* self, GnomenuItemState value);
const char* gnomenu_item_get_item_label (GnomenuItem* self);
void gnomenu_item_set_item_label (GnomenuItem* self, const char* value);
const char* gnomenu_item_get_item_icon (GnomenuItem* self);
void gnomenu_item_set_item_icon (GnomenuItem* self, const char* value);
const char* gnomenu_item_get_item_accel_text (GnomenuItem* self);
void gnomenu_item_set_item_accel_text (GnomenuItem* self, const char* value);
const char* gnomenu_item_get_item_font (GnomenuItem* self);
void gnomenu_item_set_item_font (GnomenuItem* self, const char* value);
char* gnomenu_item_get_item_path_name (GnomenuItem* self);
char* gnomenu_item_get_item_path (GnomenuItem* self);




GType gnomenu_item_type_get_type (void) {
	static GType gnomenu_item_type_type_id = 0;
	if (G_UNLIKELY (gnomenu_item_type_type_id == 0)) {
		static const GEnumValue values[] = {{GNOMENU_ITEM_TYPE_NORMAL, "GNOMENU_ITEM_TYPE_NORMAL", "normal"}, {GNOMENU_ITEM_TYPE_CHECK, "GNOMENU_ITEM_TYPE_CHECK", "check"}, {GNOMENU_ITEM_TYPE_RADIO, "GNOMENU_ITEM_TYPE_RADIO", "radio"}, {GNOMENU_ITEM_TYPE_IMAGE, "GNOMENU_ITEM_TYPE_IMAGE", "image"}, {GNOMENU_ITEM_TYPE_SEPARATOR, "GNOMENU_ITEM_TYPE_SEPARATOR", "separator"}, {GNOMENU_ITEM_TYPE_ARROW, "GNOMENU_ITEM_TYPE_ARROW", "arrow"}, {GNOMENU_ITEM_TYPE_ICON, "GNOMENU_ITEM_TYPE_ICON", "icon"}, {0, NULL, NULL}};
		gnomenu_item_type_type_id = g_enum_register_static ("GnomenuItemType", values);
	}
	return gnomenu_item_type_type_id;
}



GType gnomenu_item_state_get_type (void) {
	static GType gnomenu_item_state_type_id = 0;
	if (G_UNLIKELY (gnomenu_item_state_type_id == 0)) {
		static const GEnumValue values[] = {{GNOMENU_ITEM_STATE_UNTOGGLED, "GNOMENU_ITEM_STATE_UNTOGGLED", "untoggled"}, {GNOMENU_ITEM_STATE_TOGGLED, "GNOMENU_ITEM_STATE_TOGGLED", "toggled"}, {GNOMENU_ITEM_STATE_TRISTATE, "GNOMENU_ITEM_STATE_TRISTATE", "tristate"}, {0, NULL, NULL}};
		gnomenu_item_state_type_id = g_enum_register_static ("GnomenuItemState", values);
	}
	return gnomenu_item_state_type_id;
}


gboolean gnomenu_item_is_child_of (GnomenuItem* self, GnomenuItem* possible_parent) {
	GnomenuShell* parent_shell;
	GnomenuItem* _tmp0_;
	GnomenuItem* parent_item;
	GnomenuItem* _tmp6_;
	GnomenuItem* _tmp5_;
	GnomenuShell* _tmp4_;
	GnomenuShell* _tmp3_;
	GnomenuItem* _tmp2_;
	GnomenuItem* _tmp1_;
	gboolean _tmp8_;
	g_return_val_if_fail (possible_parent != NULL, FALSE);
	parent_shell = NULL;
	_tmp0_ = NULL;
	parent_item = (_tmp0_ = self, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp6_ = NULL;
	_tmp5_ = NULL;
	_tmp4_ = NULL;
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	for (parent_item = (_tmp2_ = (_tmp1_ = self, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (parent_item == NULL) ? NULL : (parent_item = (g_object_unref (parent_item), NULL)), _tmp2_); TRUE; parent_shell = (_tmp4_ = (_tmp3_ = gnomenu_item_get_shell (parent_item), (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (parent_shell == NULL) ? NULL : (parent_shell = (g_object_unref (parent_shell), NULL)), _tmp4_), parent_item = (_tmp6_ = (_tmp5_ = gnomenu_shell_get_owner (parent_shell), (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (parent_item == NULL) ? NULL : (parent_item = (g_object_unref (parent_item), NULL)), _tmp6_)) {
		gboolean _tmp7_;
		_tmp7_ = FALSE;
		if (parent_item != NULL) {
			_tmp7_ = parent_item != possible_parent;
		} else {
			_tmp7_ = FALSE;
		}
		if (!_tmp7_) {
			break;
		}
		continue;
	}
	return (_tmp8_ = parent_item != NULL, (parent_shell == NULL) ? NULL : (parent_shell = (g_object_unref (parent_shell), NULL)), (parent_item == NULL) ? NULL : (parent_item = (g_object_unref (parent_item), NULL)), _tmp8_);
}


GnomenuItemState gnomenu_item_state_from_string (const char* str) {
	GQuark _tmp4_;
	const char* _tmp3_;
	_tmp3_ = NULL;
	static GQuark _tmp4__label0 = 0;
	static GQuark _tmp4__label1 = 0;
	static GQuark _tmp4__label2 = 0;
	static GQuark _tmp4__label3 = 0;
	static GQuark _tmp4__label4 = 0;
	static GQuark _tmp4__label5 = 0;
	static GQuark _tmp4__label6 = 0;
	static GQuark _tmp4__label7 = 0;
	_tmp3_ = str;
	_tmp4_ = (NULL == _tmp3_) ? 0 : g_quark_from_string (_tmp3_);
	if ((((_tmp4_ == ((0 != _tmp4__label0) ? _tmp4__label0 : (_tmp4__label0 = g_quark_from_static_string ("true")))) || (_tmp4_ == ((0 != _tmp4__label1) ? _tmp4__label1 : (_tmp4__label1 = g_quark_from_static_string ("toggled"))))) || (_tmp4_ == ((0 != _tmp4__label2) ? _tmp4__label2 : (_tmp4__label2 = g_quark_from_static_string ("t"))))) || (_tmp4_ == ((0 != _tmp4__label3) ? _tmp4__label3 : (_tmp4__label3 = g_quark_from_static_string ("1")))))
	do {
		return GNOMENU_ITEM_STATE_TOGGLED;
	} while (0); else if ((((_tmp4_ == ((0 != _tmp4__label4) ? _tmp4__label4 : (_tmp4__label4 = g_quark_from_static_string ("false")))) || (_tmp4_ == ((0 != _tmp4__label5) ? _tmp4__label5 : (_tmp4__label5 = g_quark_from_static_string ("untoggled"))))) || (_tmp4_ == ((0 != _tmp4__label6) ? _tmp4__label6 : (_tmp4__label6 = g_quark_from_static_string ("f"))))) || (_tmp4_ == ((0 != _tmp4__label7) ? _tmp4__label7 : (_tmp4__label7 = g_quark_from_static_string ("0")))))
	do {
		return GNOMENU_ITEM_STATE_UNTOGGLED;
	} while (0); else
	do {
		return GNOMENU_ITEM_STATE_TRISTATE;
	} while (0);
}


const char* gnomenu_item_state_to_string (GnomenuItemState state) {
	switch (state) {
		case GNOMENU_ITEM_STATE_UNTOGGLED:
		{
			return "untoggled";
		}
		case GNOMENU_ITEM_STATE_TOGGLED:
		{
			return "toggled";
		}
		case GNOMENU_ITEM_STATE_TRISTATE:
		{
			return NULL;
		}
	}
	return NULL;
}


GnomenuItemType gnomenu_item_type_from_string (const char* str) {
	GQuark _tmp8_;
	const char* _tmp7_;
	_tmp7_ = NULL;
	static GQuark _tmp8__label0 = 0;
	static GQuark _tmp8__label1 = 0;
	static GQuark _tmp8__label2 = 0;
	static GQuark _tmp8__label3 = 0;
	static GQuark _tmp8__label4 = 0;
	static GQuark _tmp8__label5 = 0;
	static GQuark _tmp8__label6 = 0;
	static GQuark _tmp8__label7 = 0;
	static GQuark _tmp8__label8 = 0;
	static GQuark _tmp8__label9 = 0;
	static GQuark _tmp8__label10 = 0;
	_tmp7_ = str;
	_tmp8_ = (NULL == _tmp7_) ? 0 : g_quark_from_string (_tmp7_);
	if ((_tmp8_ == ((0 != _tmp8__label0) ? _tmp8__label0 : (_tmp8__label0 = g_quark_from_static_string ("check")))) || (_tmp8_ == ((0 != _tmp8__label1) ? _tmp8__label1 : (_tmp8__label1 = g_quark_from_static_string ("c")))))
	do {
		return GNOMENU_ITEM_TYPE_CHECK;
	} while (0); else if ((_tmp8_ == ((0 != _tmp8__label2) ? _tmp8__label2 : (_tmp8__label2 = g_quark_from_static_string ("radio")))) || (_tmp8_ == ((0 != _tmp8__label3) ? _tmp8__label3 : (_tmp8__label3 = g_quark_from_static_string ("r")))))
	do {
		return GNOMENU_ITEM_TYPE_RADIO;
	} while (0); else if ((_tmp8_ == ((0 != _tmp8__label4) ? _tmp8__label4 : (_tmp8__label4 = g_quark_from_static_string ("image")))) || (_tmp8_ == ((0 != _tmp8__label5) ? _tmp8__label5 : (_tmp8__label5 = g_quark_from_static_string ("i")))))
	do {
		return GNOMENU_ITEM_TYPE_IMAGE;
	} while (0); else if ((_tmp8_ == ((0 != _tmp8__label6) ? _tmp8__label6 : (_tmp8__label6 = g_quark_from_static_string ("arrow")))) || (_tmp8_ == ((0 != _tmp8__label7) ? _tmp8__label7 : (_tmp8__label7 = g_quark_from_static_string ("a")))))
	do {
		return GNOMENU_ITEM_TYPE_ARROW;
	} while (0); else if ((_tmp8_ == ((0 != _tmp8__label8) ? _tmp8__label8 : (_tmp8__label8 = g_quark_from_static_string ("separator")))) || (_tmp8_ == ((0 != _tmp8__label9) ? _tmp8__label9 : (_tmp8__label9 = g_quark_from_static_string ("s")))))
	do {
		return GNOMENU_ITEM_TYPE_SEPARATOR;
	} while (0); else if (_tmp8_ == ((0 != _tmp8__label10) ? _tmp8__label10 : (_tmp8__label10 = g_quark_from_static_string ("icon"))))
	do {
		return GNOMENU_ITEM_TYPE_ICON;
	} while (0); else
	do {
		return GNOMENU_ITEM_TYPE_NORMAL;
	} while (0);
}


gboolean gnomenu_item_type_has_label (GnomenuItemType type) {
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	if (type == GNOMENU_ITEM_TYPE_NORMAL) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = type == GNOMENU_ITEM_TYPE_IMAGE;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = type == GNOMENU_ITEM_TYPE_CHECK;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = type == GNOMENU_ITEM_TYPE_RADIO;
	}
	if (_tmp0_) {
		return TRUE;
	}
	return FALSE;
}


const char* gnomenu_item_type_to_string (GnomenuItemType type) {
	switch (type) {
		case GNOMENU_ITEM_TYPE_CHECK:
		{
			return "check";
		}
		case GNOMENU_ITEM_TYPE_RADIO:
		{
			return "radio";
		}
		case GNOMENU_ITEM_TYPE_NORMAL:
		{
			return NULL;
		}
		case GNOMENU_ITEM_TYPE_IMAGE:
		{
			return "image";
		}
		case GNOMENU_ITEM_TYPE_ICON:
		{
			return "icon";
		}
		case GNOMENU_ITEM_TYPE_ARROW:
		{
			return "arrow";
		}
		case GNOMENU_ITEM_TYPE_SEPARATOR:
		{
			return "separator";
		}
	}
	return NULL;
}


GnomenuShell* gnomenu_item_get_shell (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_shell (self);
}


GnomenuShell* gnomenu_item_get_sub_shell (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_sub_shell (self);
}


gboolean gnomenu_item_get_has_sub_shell (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_has_sub_shell (self);
}


void gnomenu_item_set_has_sub_shell (GnomenuItem* self, gboolean value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_has_sub_shell (self, value);
}


gboolean gnomenu_item_get_client_side_sub_shell (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_client_side_sub_shell (self);
}


void gnomenu_item_set_client_side_sub_shell (GnomenuItem* self, gboolean value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_client_side_sub_shell (self, value);
}


GnomenuShell* gnomenu_item_get_toplevel_shell (GnomenuItem* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return gnomenu_shell_get_toplevel_shell (gnomenu_item_get_shell (self));
}


gint gnomenu_item_get_item_position (GnomenuItem* self) {
	g_return_val_if_fail (self != NULL, 0);
	return gnomenu_shell_get_item_position (gnomenu_item_get_shell (self), self);
}


const char* gnomenu_item_get_item_id (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_id (self);
}


void gnomenu_item_set_item_id (GnomenuItem* self, const char* value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_id (self, value);
}


GnomenuItemType gnomenu_item_get_item_type (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_type (self);
}


void gnomenu_item_set_item_type (GnomenuItem* self, GnomenuItemType value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_type (self, value);
}


gboolean gnomenu_item_get_item_use_underline (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_use_underline (self);
}


void gnomenu_item_set_item_use_underline (GnomenuItem* self, gboolean value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_use_underline (self, value);
}


gboolean gnomenu_item_get_item_sensitive (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_sensitive (self);
}


void gnomenu_item_set_item_sensitive (GnomenuItem* self, gboolean value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_sensitive (self, value);
}


gboolean gnomenu_item_get_item_visible (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_visible (self);
}


void gnomenu_item_set_item_visible (GnomenuItem* self, gboolean value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_visible (self, value);
}


GnomenuItemState gnomenu_item_get_item_state (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_state (self);
}


void gnomenu_item_set_item_state (GnomenuItem* self, GnomenuItemState value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_state (self, value);
}


const char* gnomenu_item_get_item_label (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_label (self);
}


void gnomenu_item_set_item_label (GnomenuItem* self, const char* value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_label (self, value);
}


const char* gnomenu_item_get_item_icon (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_icon (self);
}


void gnomenu_item_set_item_icon (GnomenuItem* self, const char* value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_icon (self, value);
}


const char* gnomenu_item_get_item_accel_text (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_accel_text (self);
}


void gnomenu_item_set_item_accel_text (GnomenuItem* self, const char* value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_accel_text (self, value);
}


const char* gnomenu_item_get_item_font (GnomenuItem* self) {
	return GNOMENU_ITEM_GET_INTERFACE (self)->get_item_font (self);
}


void gnomenu_item_set_item_font (GnomenuItem* self, const char* value) {
	GNOMENU_ITEM_GET_INTERFACE (self)->set_item_font (self, value);
}


char* gnomenu_item_get_item_path_name (GnomenuItem* self) {
	g_return_val_if_fail (self != NULL, NULL);
	if (gnomenu_item_get_item_id (self) != NULL) {
		const char* _tmp0_;
		_tmp0_ = NULL;
		return (_tmp0_ = gnomenu_item_get_item_id (self), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	} else {
		return g_strdup_printf ("%i", gnomenu_item_get_item_position (self));
	}
}


char* gnomenu_item_get_item_path (GnomenuItem* self) {
	GString* sb;
	GnomenuItem* _tmp0_;
	GnomenuItem* item;
	GnomenuShell* _tmp1_;
	GnomenuShell* parent;
	char* _tmp2_;
	const char* _tmp9_;
	char* _tmp10_;
	g_return_val_if_fail (self != NULL, NULL);
	sb = g_string_new ("");
	_tmp0_ = NULL;
	item = (_tmp0_ = self, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp1_ = NULL;
	parent = (_tmp1_ = gnomenu_item_get_shell (item), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
	_tmp2_ = NULL;
	g_string_append (sb, _tmp2_ = gnomenu_item_get_item_path_name (self));
	_tmp2_ = (g_free (_tmp2_), NULL);
	while (parent != NULL) {
		GnomenuItem* _tmp5_;
		GnomenuItem* _tmp4_;
		GnomenuItem* _tmp3_;
		char* _tmp6_;
		GnomenuShell* _tmp8_;
		GnomenuShell* _tmp7_;
		_tmp5_ = NULL;
		_tmp4_ = NULL;
		_tmp3_ = NULL;
		item = (_tmp5_ = (_tmp4_ = (_tmp3_ = gnomenu_shell_get_owner (parent), GNOMENU_IS_ITEM (_tmp3_) ? ((GnomenuItem*) _tmp3_) : NULL), (_tmp4_ == NULL) ? NULL : g_object_ref (_tmp4_)), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), _tmp5_);
		if (item == NULL) {
			break;
		}
		g_string_prepend_unichar (sb, (gunichar) '/');
		_tmp6_ = NULL;
		g_string_prepend (sb, _tmp6_ = gnomenu_item_get_item_path_name (item));
		_tmp6_ = (g_free (_tmp6_), NULL);
		_tmp8_ = NULL;
		_tmp7_ = NULL;
		parent = (_tmp8_ = (_tmp7_ = gnomenu_item_get_shell (item), (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp8_);
	}
	g_string_prepend_unichar (sb, (gunichar) '/');
	_tmp9_ = NULL;
	_tmp10_ = NULL;
	return (_tmp10_ = (_tmp9_ = sb->str, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), (sb == NULL) ? NULL : (sb = (g_string_free (sb, TRUE), NULL)), (item == NULL) ? NULL : (item = (g_object_unref (item), NULL)), (parent == NULL) ? NULL : (parent = (g_object_unref (parent), NULL)), _tmp10_);
}


static void gnomenu_item_base_init (GnomenuItemIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
		g_object_interface_install_property (iface, g_param_spec_object ("shell", "shell", "shell", GNOMENU_TYPE_SHELL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_object ("sub-shell", "sub-shell", "sub-shell", GNOMENU_TYPE_SHELL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_boolean ("has-sub-shell", "has-sub-shell", "has-sub-shell", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_boolean ("client-side-sub-shell", "client-side-sub-shell", "client-side-sub-shell", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("item-id", "item-id", "item-id", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_enum ("item-type", "item-type", "item-type", GNOMENU_TYPE_ITEM_TYPE, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_boolean ("item-use-underline", "item-use-underline", "item-use-underline", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_boolean ("item-sensitive", "item-sensitive", "item-sensitive", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_boolean ("item-visible", "item-visible", "item-visible", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_enum ("item-state", "item-state", "item-state", GNOMENU_TYPE_ITEM_STATE, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("item-label", "item-label", "item-label", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("item-icon", "item-icon", "item-icon", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("item-accel-text", "item-accel-text", "item-accel-text", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("item-font", "item-font", "item-font", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	}
}


GType gnomenu_item_get_type (void) {
	static GType gnomenu_item_type_id = 0;
	if (gnomenu_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuItemIface), (GBaseInitFunc) gnomenu_item_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		gnomenu_item_type_id = g_type_register_static (G_TYPE_INTERFACE, "GnomenuItem", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gnomenu_item_type_id, G_TYPE_OBJECT);
	}
	return gnomenu_item_type_id;
}




