
#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define GNOMENU_TYPE_SHELL (gnomenu_shell_get_type ())
#define GNOMENU_SHELL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_SHELL, GnomenuShell))
#define GNOMENU_IS_SHELL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_SHELL))
#define GNOMENU_SHELL_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GNOMENU_TYPE_SHELL, GnomenuShellIface))

typedef struct _GnomenuShell GnomenuShell;
typedef struct _GnomenuShellIface GnomenuShellIface;

#define GNOMENU_TYPE_ITEM (gnomenu_item_get_type ())
#define GNOMENU_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_ITEM, GnomenuItem))
#define GNOMENU_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_ITEM))
#define GNOMENU_ITEM_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GNOMENU_TYPE_ITEM, GnomenuItemIface))

typedef struct _GnomenuItem GnomenuItem;
typedef struct _GnomenuItemIface GnomenuItemIface;

#define GNOMENU_TYPE_ITEM_TYPE (gnomenu_item_type_get_type ())

#define GNOMENU_TYPE_ITEM_STATE (gnomenu_item_state_get_type ())

typedef enum  {
	GNOMENU_ITEM_TYPE_NORMAL = 0,
	GNOMENU_ITEM_TYPE_CHECK = 1,
	GNOMENU_ITEM_TYPE_RADIO = 2,
	GNOMENU_ITEM_TYPE_IMAGE = 3,
	GNOMENU_ITEM_TYPE_SEPARATOR = 4,
	GNOMENU_ITEM_TYPE_ARROW = 5,
	GNOMENU_ITEM_TYPE_ICON = 6
} GnomenuItemType;

typedef enum  {
	GNOMENU_ITEM_STATE_UNTOGGLED = 0,
	GNOMENU_ITEM_STATE_TOGGLED = 1,
	GNOMENU_ITEM_STATE_TRISTATE = 2
} GnomenuItemState;

struct _GnomenuItemIface {
	GTypeInterface parent_iface;
	GnomenuShell* (*get_shell) (GnomenuItem* self);
	GnomenuShell* (*get_sub_shell) (GnomenuItem* self);
	gboolean (*get_has_sub_shell) (GnomenuItem* self);
	void (*set_has_sub_shell) (GnomenuItem* self, gboolean value);
	gboolean (*get_client_side_sub_shell) (GnomenuItem* self);
	void (*set_client_side_sub_shell) (GnomenuItem* self, gboolean value);
	const char* (*get_item_id) (GnomenuItem* self);
	void (*set_item_id) (GnomenuItem* self, const char* value);
	GnomenuItemType (*get_item_type) (GnomenuItem* self);
	void (*set_item_type) (GnomenuItem* self, GnomenuItemType value);
	gboolean (*get_item_use_underline) (GnomenuItem* self);
	void (*set_item_use_underline) (GnomenuItem* self, gboolean value);
	gboolean (*get_item_sensitive) (GnomenuItem* self);
	void (*set_item_sensitive) (GnomenuItem* self, gboolean value);
	gboolean (*get_item_visible) (GnomenuItem* self);
	void (*set_item_visible) (GnomenuItem* self, gboolean value);
	GnomenuItemState (*get_item_state) (GnomenuItem* self);
	void (*set_item_state) (GnomenuItem* self, GnomenuItemState value);
	const char* (*get_item_label) (GnomenuItem* self);
	void (*set_item_label) (GnomenuItem* self, const char* value);
	const char* (*get_item_icon) (GnomenuItem* self);
	void (*set_item_icon) (GnomenuItem* self, const char* value);
	const char* (*get_item_accel_text) (GnomenuItem* self);
	void (*set_item_accel_text) (GnomenuItem* self, const char* value);
	const char* (*get_item_font) (GnomenuItem* self);
	void (*set_item_font) (GnomenuItem* self, const char* value);
};

struct _GnomenuShellIface {
	GTypeInterface parent_iface;
	GnomenuItem* (*get_item) (GnomenuShell* self, gint position);
	GnomenuItem* (*get_item_by_id) (GnomenuShell* self, const char* id);
	gint (*get_item_position) (GnomenuShell* self, GnomenuItem* item);
	GnomenuItem* (*get_owner) (GnomenuShell* self);
	gint (*get_length) (GnomenuShell* self);
	void (*set_length) (GnomenuShell* self, gint value);
};



GType gnomenu_item_type_get_type (void);
GType gnomenu_item_state_get_type (void);
GType gnomenu_item_get_type (void);
GType gnomenu_shell_get_type (void);
GnomenuItem* gnomenu_shell_get_item (GnomenuShell* self, gint position);
GnomenuItem* gnomenu_shell_get_item_by_id (GnomenuShell* self, const char* id);
gint gnomenu_shell_get_item_position (GnomenuShell* self, GnomenuItem* item);
GnomenuItem* gnomenu_shell_get_owner (GnomenuShell* self);
gint gnomenu_shell_get_length (GnomenuShell* self);
void gnomenu_shell_set_length (GnomenuShell* self, gint value);
GnomenuShell* gnomenu_item_get_toplevel_shell (GnomenuItem* self);
GnomenuShell* gnomenu_shell_get_toplevel_shell (GnomenuShell* self);



GnomenuItem* gnomenu_shell_get_item (GnomenuShell* self, gint position) {
	return GNOMENU_SHELL_GET_INTERFACE (self)->get_item (self, position);
}


GnomenuItem* gnomenu_shell_get_item_by_id (GnomenuShell* self, const char* id) {
	return GNOMENU_SHELL_GET_INTERFACE (self)->get_item_by_id (self, id);
}


gint gnomenu_shell_get_item_position (GnomenuShell* self, GnomenuItem* item) {
	return GNOMENU_SHELL_GET_INTERFACE (self)->get_item_position (self, item);
}


GnomenuItem* gnomenu_shell_get_owner (GnomenuShell* self) {
	return GNOMENU_SHELL_GET_INTERFACE (self)->get_owner (self);
}


gint gnomenu_shell_get_length (GnomenuShell* self) {
	return GNOMENU_SHELL_GET_INTERFACE (self)->get_length (self);
}


void gnomenu_shell_set_length (GnomenuShell* self, gint value) {
	GNOMENU_SHELL_GET_INTERFACE (self)->set_length (self, value);
}


GnomenuShell* gnomenu_shell_get_toplevel_shell (GnomenuShell* self) {
	g_return_val_if_fail (self != NULL, NULL);
	if (gnomenu_shell_get_owner (self) != NULL) {
		return gnomenu_item_get_toplevel_shell (gnomenu_shell_get_owner (self));
	}
	return self;
}


static void gnomenu_shell_base_init (GnomenuShellIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
		g_object_interface_install_property (iface, g_param_spec_object ("owner", "owner", "owner", GNOMENU_TYPE_ITEM, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_int ("length", "length", "length", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	}
}


GType gnomenu_shell_get_type (void) {
	static GType gnomenu_shell_type_id = 0;
	if (gnomenu_shell_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuShellIface), (GBaseInitFunc) gnomenu_shell_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		gnomenu_shell_type_id = g_type_register_static (G_TYPE_INTERFACE, "GnomenuShell", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gnomenu_shell_type_id, G_TYPE_OBJECT);
	}
	return gnomenu_shell_type_id;
}




