
#include <glib.h>
#include <glib-object.h>
#include <gdk/gdk.h>
#include <libwnck/libwnck.h>


#define GNOMENU_TYPE_MONITOR (gnomenu_monitor_get_type ())
#define GNOMENU_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOMENU_TYPE_MONITOR, GnomenuMonitor))
#define GNOMENU_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GNOMENU_TYPE_MONITOR, GnomenuMonitorClass))
#define GNOMENU_IS_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOMENU_TYPE_MONITOR))
#define GNOMENU_IS_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOMENU_TYPE_MONITOR))
#define GNOMENU_MONITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GNOMENU_TYPE_MONITOR, GnomenuMonitorClass))

typedef struct _GnomenuMonitor GnomenuMonitor;
typedef struct _GnomenuMonitorClass GnomenuMonitorClass;
typedef struct _GnomenuMonitorPrivate GnomenuMonitorPrivate;

struct _GnomenuMonitor {
	GObject parent_instance;
	GnomenuMonitorPrivate * priv;
};

struct _GnomenuMonitorClass {
	GObjectClass parent_class;
};

struct _GnomenuMonitorPrivate {
	GdkScreen* _screen;
	WnckScreen* _wnck_screen;
	WnckWindow* _desktop;
	WnckWindow* _current_window;
	gboolean disposed;
};



GType gnomenu_monitor_get_type (void);
#define GNOMENU_MONITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GNOMENU_TYPE_MONITOR, GnomenuMonitorPrivate))
enum  {
	GNOMENU_MONITOR_DUMMY_PROPERTY,
	GNOMENU_MONITOR_SCREEN,
	GNOMENU_MONITOR_CURRENT_XID
};
static WnckScreen* gnomenu_monitor_gdk_to_wnck_screen (GnomenuMonitor* self, GdkScreen* screen);
GnomenuMonitor* gnomenu_monitor_new (void);
GnomenuMonitor* gnomenu_monitor_construct (GType object_type);
GnomenuMonitor* gnomenu_monitor_new (void);
static void gnomenu_monitor_detach_from_screen (GnomenuMonitor* self, WnckScreen* screen);
static void gnomenu_monitor_real_dispose (GObject* base);
static void gnomenu_monitor_update_current_window (GnomenuMonitor* self);
static void gnomenu_monitor_on_window_closed (GnomenuMonitor* self, WnckScreen* screen, WnckWindow* window);
static void gnomenu_monitor_on_window_opened (GnomenuMonitor* self, WnckScreen* screen, WnckWindow* window);
static void gnomenu_monitor_on_active_window_changed (GnomenuMonitor* self, WnckScreen* screen, WnckWindow* previous_window);
static WnckWindow* gnomenu_monitor_find_desktop (GnomenuMonitor* self, WnckScreen* screen);
static void _gnomenu_monitor_on_window_opened_wnck_screen_window_opened (WnckScreen* _sender, WnckWindow* window, gpointer self);
static void _gnomenu_monitor_on_window_closed_wnck_screen_window_closed (WnckScreen* _sender, WnckWindow* window, gpointer self);
static void _gnomenu_monitor_on_active_window_changed_wnck_screen_active_window_changed (WnckScreen* _sender, WnckWindow* previous_window, gpointer self);
static void gnomenu_monitor_attach_to_screen (GnomenuMonitor* self, WnckScreen* screen);
GdkScreen* gnomenu_monitor_get_screen (GnomenuMonitor* self);
void gnomenu_monitor_set_screen (GnomenuMonitor* self, GdkScreen* value);
gulong gnomenu_monitor_get_current_xid (GnomenuMonitor* self);
static GObject * gnomenu_monitor_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer gnomenu_monitor_parent_class = NULL;
static void gnomenu_monitor_finalize (GObject* obj);
static void gnomenu_monitor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void gnomenu_monitor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);



static WnckScreen* gnomenu_monitor_gdk_to_wnck_screen (GnomenuMonitor* self, GdkScreen* screen) {
	g_return_val_if_fail (self != NULL, NULL);
	if (screen != NULL) {
		WnckScreen* _tmp0_;
		_tmp0_ = NULL;
		return (_tmp0_ = wnck_screen_get (gdk_screen_get_number (screen)), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	}
	return NULL;
}


GnomenuMonitor* gnomenu_monitor_construct (GType object_type) {
	GnomenuMonitor * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


GnomenuMonitor* gnomenu_monitor_new (void) {
	return gnomenu_monitor_construct (GNOMENU_TYPE_MONITOR);
}


static void gnomenu_monitor_real_dispose (GObject* base) {
	GnomenuMonitor * self;
	self = (GnomenuMonitor*) base;
	if (!self->priv->disposed) {
		self->priv->disposed = TRUE;
		gnomenu_monitor_detach_from_screen (self, self->priv->_wnck_screen);
	}
}


static void gnomenu_monitor_on_window_closed (GnomenuMonitor* self, WnckScreen* screen, WnckWindow* window) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	g_return_if_fail (window != NULL);
	if (self->priv->_desktop == window) {
		WnckWindow* _tmp0_;
		_tmp0_ = NULL;
		self->priv->_desktop = (_tmp0_ = NULL, (self->priv->_desktop == NULL) ? NULL : (self->priv->_desktop = (g_object_unref (self->priv->_desktop), NULL)), _tmp0_);
	}
	if (window == self->priv->_current_window) {
		g_object_set_data ((GObject*) window, "window-closed", window);
		gnomenu_monitor_update_current_window (self);
	}
}


static void gnomenu_monitor_on_window_opened (GnomenuMonitor* self, WnckScreen* screen, WnckWindow* window) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	g_return_if_fail (window != NULL);
	if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP) {
		WnckWindow* _tmp1_;
		WnckWindow* _tmp0_;
		_tmp1_ = NULL;
		_tmp0_ = NULL;
		self->priv->_desktop = (_tmp1_ = (_tmp0_ = window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_desktop == NULL) ? NULL : (self->priv->_desktop = (g_object_unref (self->priv->_desktop), NULL)), _tmp1_);
	}
	if (self->priv->_current_window == NULL) {
		gnomenu_monitor_update_current_window (self);
	}
}


static void gnomenu_monitor_on_active_window_changed (GnomenuMonitor* self, WnckScreen* screen, WnckWindow* previous_window) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	gnomenu_monitor_update_current_window (self);
}


static WnckWindow* gnomenu_monitor_find_desktop (GnomenuMonitor* self, WnckScreen* screen) {
	GList* windows;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (screen != NULL, NULL);
	windows = wnck_screen_get_windows (screen);
	{
		GList* window_collection;
		GList* window_it;
		window_collection = windows;
		for (window_it = window_collection; window_it != NULL; window_it = window_it->next) {
			WnckWindow* window;
			window = (WnckWindow*) window_it->data;
			{
				if (wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP) {
					WnckWindow* _tmp0_;
					/*Vala should ref it*/
					_tmp0_ = NULL;
					return (_tmp0_ = window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
				}
			}
		}
	}
	return NULL;
}


static void gnomenu_monitor_update_current_window (GnomenuMonitor* self) {
	WnckWindow* _tmp0_;
	WnckWindow* old;
	WnckWindow* _tmp2_;
	WnckWindow* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = NULL;
	old = (_tmp0_ = self->priv->_current_window, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
	_tmp2_ = NULL;
	_tmp1_ = NULL;
	self->priv->_current_window = (_tmp2_ = (_tmp1_ = wnck_screen_get_active_window (self->priv->_wnck_screen), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL)), _tmp2_);
	if (self->priv->_current_window == NULL) {
		WnckWindow* _tmp4_;
		WnckWindow* _tmp3_;
		_tmp4_ = NULL;
		_tmp3_ = NULL;
		self->priv->_current_window = (_tmp4_ = (_tmp3_ = self->priv->_desktop, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL)), _tmp4_);
	}
	if (self->priv->_current_window != NULL) {
		WnckWindow* transient_for;
		transient_for = wnck_window_get_transient (self->priv->_current_window);
		if (transient_for != NULL) {
			WnckWindow* _tmp6_;
			WnckWindow* _tmp5_;
			_tmp6_ = NULL;
			_tmp5_ = NULL;
			self->priv->_current_window = (_tmp6_ = (_tmp5_ = transient_for, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL)), _tmp6_);
		}
		switch (wnck_window_get_window_type (self->priv->_current_window)) {
			case WNCK_WINDOW_NORMAL:
			case WNCK_WINDOW_DESKTOP:
			case WNCK_WINDOW_UTILITY:
			case WNCK_WINDOW_DIALOG:
			{
				break;
			}
			default:
			{
				WnckWindow* _tmp8_;
				WnckWindow* _tmp7_;
				_tmp8_ = NULL;
				_tmp7_ = NULL;
				self->priv->_current_window = (_tmp8_ = (_tmp7_ = self->priv->_desktop, (_tmp7_ == NULL) ? NULL : g_object_ref (_tmp7_)), (self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL)), _tmp8_);
				g_object_set_data ((GObject*) old, "window-closed", NULL);
				break;
			}
		}
	}
	if (old == self->priv->_current_window) {
		(old == NULL) ? NULL : (old = (g_object_unref (old), NULL));
		return;
	}
	if (old != NULL) {
		g_signal_emit_by_name (self, "window-changed", wnck_window_get_xid (old));
	} else {
		g_signal_emit_by_name (self, "window-changed", (gulong) 0);
	}
	(old == NULL) ? NULL : (old = (g_object_unref (old), NULL));
}


static void _gnomenu_monitor_on_window_opened_wnck_screen_window_opened (WnckScreen* _sender, WnckWindow* window, gpointer self) {
	gnomenu_monitor_on_window_opened (self, _sender, window);
}


static void _gnomenu_monitor_on_window_closed_wnck_screen_window_closed (WnckScreen* _sender, WnckWindow* window, gpointer self) {
	gnomenu_monitor_on_window_closed (self, _sender, window);
}


static void _gnomenu_monitor_on_active_window_changed_wnck_screen_active_window_changed (WnckScreen* _sender, WnckWindow* previous_window, gpointer self) {
	gnomenu_monitor_on_active_window_changed (self, _sender, previous_window);
}


static void gnomenu_monitor_detach_from_screen (GnomenuMonitor* self, WnckScreen* screen) {
	guint _tmp0_;
	guint _tmp1_;
	guint _tmp2_;
	WnckWindow* _tmp3_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	g_signal_handlers_disconnect_matched (screen, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("window-opened", WNCK_TYPE_SCREEN, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _gnomenu_monitor_on_window_opened_wnck_screen_window_opened, self);
	g_signal_handlers_disconnect_matched (screen, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("window-closed", WNCK_TYPE_SCREEN, &_tmp1_, NULL, FALSE), _tmp1_), 0, NULL, (GCallback) _gnomenu_monitor_on_window_closed_wnck_screen_window_closed, self);
	g_signal_handlers_disconnect_matched (screen, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("active-window-changed", WNCK_TYPE_SCREEN, &_tmp2_, NULL, FALSE), _tmp2_), 0, NULL, (GCallback) _gnomenu_monitor_on_active_window_changed_wnck_screen_active_window_changed, self);
	_tmp3_ = NULL;
	self->priv->_desktop = (_tmp3_ = NULL, (self->priv->_desktop == NULL) ? NULL : (self->priv->_desktop = (g_object_unref (self->priv->_desktop), NULL)), _tmp3_);
}


static void gnomenu_monitor_attach_to_screen (GnomenuMonitor* self, WnckScreen* screen) {
	WnckWindow* new_desktop;
	WnckWindow* _tmp1_;
	WnckWindow* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (screen != NULL);
	g_signal_connect_object (screen, "window-closed", (GCallback) _gnomenu_monitor_on_window_closed_wnck_screen_window_closed, self, 0);
	g_signal_connect_object (screen, "window-opened", (GCallback) _gnomenu_monitor_on_window_opened_wnck_screen_window_opened, self, 0);
	g_signal_connect_object (screen, "active-window-changed", (GCallback) _gnomenu_monitor_on_active_window_changed_wnck_screen_active_window_changed, self, 0);
	new_desktop = gnomenu_monitor_find_desktop (self, screen);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_desktop = (_tmp1_ = (_tmp0_ = new_desktop, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_desktop == NULL) ? NULL : (self->priv->_desktop = (g_object_unref (self->priv->_desktop), NULL)), _tmp1_);
	(new_desktop == NULL) ? NULL : (new_desktop = (g_object_unref (new_desktop), NULL));
}


GdkScreen* gnomenu_monitor_get_screen (GnomenuMonitor* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_screen;
}


void gnomenu_monitor_set_screen (GnomenuMonitor* self, GdkScreen* value) {
	WnckScreen* new_screen;
	GdkScreen* _tmp4_;
	GdkScreen* _tmp3_;
	WnckScreen* _tmp6_;
	WnckScreen* _tmp5_;
	g_return_if_fail (self != NULL);
	new_screen = gnomenu_monitor_gdk_to_wnck_screen (self, value);
	if (new_screen == NULL) {
		WnckScreen* _tmp2_;
		WnckScreen* _tmp1_;
		_tmp2_ = NULL;
		_tmp1_ = NULL;
		new_screen = (_tmp2_ = (_tmp1_ = wnck_screen_get_default (), (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_)), (new_screen == NULL) ? NULL : (new_screen = (g_object_unref (new_screen), NULL)), _tmp2_);
	}
	if (new_screen == self->priv->_wnck_screen) {
		(new_screen == NULL) ? NULL : (new_screen = (g_object_unref (new_screen), NULL));
		return;
	}
	_tmp4_ = NULL;
	_tmp3_ = NULL;
	self->priv->_screen = (_tmp4_ = (_tmp3_ = value, (_tmp3_ == NULL) ? NULL : g_object_ref (_tmp3_)), (self->priv->_screen == NULL) ? NULL : (self->priv->_screen = (g_object_unref (self->priv->_screen), NULL)), _tmp4_);
	if (self->priv->_wnck_screen != NULL) {
		gnomenu_monitor_detach_from_screen (self, self->priv->_wnck_screen);
	}
	_tmp6_ = NULL;
	_tmp5_ = NULL;
	self->priv->_wnck_screen = (_tmp6_ = (_tmp5_ = new_screen, (_tmp5_ == NULL) ? NULL : g_object_ref (_tmp5_)), (self->priv->_wnck_screen == NULL) ? NULL : (self->priv->_wnck_screen = (g_object_unref (self->priv->_wnck_screen), NULL)), _tmp6_);
	if (self->priv->_wnck_screen != NULL) {
		gnomenu_monitor_attach_to_screen (self, self->priv->_wnck_screen);
		/* sync to wnck status, may invoke a whole bunch of
		 * signals */
		wnck_screen_force_update (self->priv->_wnck_screen);
	}
	(new_screen == NULL) ? NULL : (new_screen = (g_object_unref (new_screen), NULL));
	g_object_notify ((GObject *) self, "screen");
}


gulong gnomenu_monitor_get_current_xid (GnomenuMonitor* self) {
	g_return_val_if_fail (self != NULL, 0UL);
	if (self->priv->_current_window != NULL) {
		return wnck_window_get_xid (self->priv->_current_window);
	}
	return (gulong) 0;
}


static GObject * gnomenu_monitor_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GnomenuMonitorClass * klass;
	GObjectClass * parent_class;
	GnomenuMonitor * self;
	klass = GNOMENU_MONITOR_CLASS (g_type_class_peek (GNOMENU_TYPE_MONITOR));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GNOMENU_MONITOR (obj);
	{
		gnomenu_monitor_set_screen (self, gdk_screen_get_default ());
		self->priv->disposed = FALSE;
	}
	return obj;
}


static void gnomenu_monitor_class_init (GnomenuMonitorClass * klass) {
	gnomenu_monitor_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GnomenuMonitorPrivate));
	G_OBJECT_CLASS (klass)->dispose = gnomenu_monitor_real_dispose;
	G_OBJECT_CLASS (klass)->get_property = gnomenu_monitor_get_property;
	G_OBJECT_CLASS (klass)->set_property = gnomenu_monitor_set_property;
	G_OBJECT_CLASS (klass)->constructor = gnomenu_monitor_constructor;
	G_OBJECT_CLASS (klass)->finalize = gnomenu_monitor_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MONITOR_SCREEN, g_param_spec_object ("screen", "screen", "screen", GDK_TYPE_SCREEN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GNOMENU_MONITOR_CURRENT_XID, g_param_spec_ulong ("current-xid", "current-xid", "current-xid", 0, G_MAXULONG, 0UL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("window_changed", GNOMENU_TYPE_MONITOR, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__ULONG, G_TYPE_NONE, 1, G_TYPE_ULONG);
}


static void gnomenu_monitor_instance_init (GnomenuMonitor * self) {
	self->priv = GNOMENU_MONITOR_GET_PRIVATE (self);
}


static void gnomenu_monitor_finalize (GObject* obj) {
	GnomenuMonitor * self;
	self = GNOMENU_MONITOR (obj);
	(self->priv->_screen == NULL) ? NULL : (self->priv->_screen = (g_object_unref (self->priv->_screen), NULL));
	(self->priv->_wnck_screen == NULL) ? NULL : (self->priv->_wnck_screen = (g_object_unref (self->priv->_wnck_screen), NULL));
	(self->priv->_desktop == NULL) ? NULL : (self->priv->_desktop = (g_object_unref (self->priv->_desktop), NULL));
	(self->priv->_current_window == NULL) ? NULL : (self->priv->_current_window = (g_object_unref (self->priv->_current_window), NULL));
	G_OBJECT_CLASS (gnomenu_monitor_parent_class)->finalize (obj);
}


GType gnomenu_monitor_get_type (void) {
	static GType gnomenu_monitor_type_id = 0;
	if (gnomenu_monitor_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GnomenuMonitorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gnomenu_monitor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GnomenuMonitor), 0, (GInstanceInitFunc) gnomenu_monitor_instance_init, NULL };
		gnomenu_monitor_type_id = g_type_register_static (G_TYPE_OBJECT, "GnomenuMonitor", &g_define_type_info, 0);
	}
	return gnomenu_monitor_type_id;
}


static void gnomenu_monitor_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GnomenuMonitor * self;
	gpointer boxed;
	self = GNOMENU_MONITOR (object);
	switch (property_id) {
		case GNOMENU_MONITOR_SCREEN:
		g_value_set_object (value, gnomenu_monitor_get_screen (self));
		break;
		case GNOMENU_MONITOR_CURRENT_XID:
		g_value_set_ulong (value, gnomenu_monitor_get_current_xid (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gnomenu_monitor_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GnomenuMonitor * self;
	self = GNOMENU_MONITOR (object);
	switch (property_id) {
		case GNOMENU_MONITOR_SCREEN:
		gnomenu_monitor_set_screen (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}




